@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,22 +28,16 @@
  */
 
 #include <utility>
-#include <stdexcept>
-#include <sstream>
-#include <memory>
 #include <cstdio>
-#include "dns.h"
-#include "ip_address.h"
-#include "ipv6_address.h"
-#include "exceptions.h"
-#include "rawpdu.h"
-#include "endianness.h"
-#include "memory_helpers.h"
+#include <tins/dns.h>
+#include <tins/ip_address.h>
+#include <tins/ipv6_address.h>
+#include <tins/exceptions.h>
+#include <tins/endianness.h>
+#include <tins/memory_helpers.h>
 
 using std::string;
-using std::copy;
 using std::memcpy;
-using std::list;
 using std::make_pair;
 
 using Tins::Memory::InputMemoryStream;
@@ -51,8 +45,8 @@ using Tins::Memory::OutputMemoryStream;
 
 namespace Tins {
 
-PDU::metadata DNS::extract_metadata(const uint8_t *buffer, uint32_t total_sz) {
-    if (TINS_UNLIKELY(sizeof(dns_header))) {
+PDU::metadata DNS::extract_metadata(const uint8_t* /*buffer*/, uint32_t total_sz) {
+    if (TINS_UNLIKELY(total_sz < sizeof(dns_header))) {
         throw malformed_packet();
     }
     return metadata(total_sz, pdu_flag, PDU::UNKNOWN);
@@ -111,7 +105,7 @@ void DNS::skip_to_section_end(InputMemoryStream& stream,
         skip_to_dname_end(stream);
         stream.skip(sizeof(uint16_t) * 2 + sizeof(uint32_t));
         uint16_t data_size = stream.read_be<uint16_t>();
-        if (!stream.can_read(data_size)) {
+        if (TINS_UNLIKELY(!stream.can_read(data_size))) {
             throw malformed_packet();
         }
         stream.skip(data_size);
@@ -287,8 +281,8 @@ void DNS::add_additional(const resource& resource){
 
 string DNS::encode_domain_name(const string& dn) {
     string output;
-    size_t last_index(0), index;
     if (!dn.empty()) {
+        size_t last_index(0), index;
         while ((index = dn.find('.', last_index+1)) != string::npos) {
             output.push_back(static_cast<char>(index - last_index));
             output.append(dn.begin() + last_index, dn.begin() + index);
@@ -345,7 +339,7 @@ uint32_t DNS::compose_name(const uint8_t* ptr, char* out_ptr) const {
     while (*ptr) {
         // It's an offset
         if ((*ptr & 0xc0)) {
-            if (ptr + sizeof(uint16_t) > end) {
+            if (TINS_UNLIKELY(ptr + sizeof(uint16_t) > end)) {
                 throw malformed_packet();
             }
             uint16_t index;
@@ -366,14 +360,14 @@ uint32_t DNS::compose_name(const uint8_t* ptr, char* out_ptr) const {
             // It's a label, grab its size.
             uint8_t size = *ptr;
             ptr++;
-            if (ptr + size > end || current_out_ptr - out_ptr + size + 1 > 255) {
+            if (TINS_UNLIKELY(ptr + size > end || current_out_ptr - out_ptr + size + 1 > 255)) {
                 throw malformed_packet();
             }
             // Append a dot if it's not the first one.
             if (current_out_ptr != out_ptr) {
                 *current_out_ptr++ = '.';
             }
-            copy(ptr, ptr + size, current_out_ptr);
+            memcpy(current_out_ptr, ptr, size);
             current_out_ptr += size;
             ptr += size;
         }
@@ -386,7 +380,7 @@ uint32_t DNS::compose_name(const uint8_t* ptr, char* out_ptr) const {
     return end_ptr - start_ptr;
 }
 
-void DNS::write_serialization(uint8_t* buffer, uint32_t total_sz, const PDU* parent) {
+void DNS::write_serialization(uint8_t* buffer, uint32_t total_sz) {
     OutputMemoryStream stream(buffer, total_sz);
     stream.write(header_);
     stream.write(records_data_.begin(), records_data_.end());
@@ -436,7 +430,7 @@ void DNS::convert_records(const uint8_t* ptr,
             preference = stream.read_be<uint16_t>();
             data_size -= sizeof(uint16_t);
         }
-        if (!stream.can_read(data_size)) {
+        if (TINS_UNLIKELY(!stream.can_read(data_size))) {
             throw malformed_packet();
         }
 
@@ -476,16 +470,27 @@ void DNS::convert_records(const uint8_t* ptr,
                 stream.skip(data_size);
                 break;
         }
-        res.push_back(
-            resource(
+        #if TINS_IS_CXX11
+            res.emplace_back(
                 dname, 
-                (used_small_buffer) ? small_addr_buf : data, 
+                (used_small_buffer) ? small_addr_buf : std::move(data), 
                 type, 
                 qclass, 
                 ttl,
                 preference
-            )
-        );
+            );
+        #else
+            res.push_back(
+                resource(
+                    dname, 
+                    (used_small_buffer) ? small_addr_buf : data, 
+                    type, 
+                    qclass, 
+                    ttl,
+                    preference
+                )
+            );
+        #endif
     }
 }
 
@@ -550,9 +555,13 @@ DNS::queries_type DNS::queries() const {
             stream.skip(compose_name(stream.pointer(), buffer));
             uint16_t query_type = stream.read_be<uint16_t>();
             uint16_t query_class = stream.read_be<uint16_t>();
-            output.push_back(
-                query(buffer, (QueryType)query_type, (QueryClass)query_class)
-            );
+            #if TINS_IS_CXX11
+                output.emplace_back(buffer, (QueryType)query_type, (QueryClass)query_class);
+            #else
+                output.push_back(
+                    query(buffer, (QueryType)query_type, (QueryClass)query_class)
+                );
+            #endif
         }
     }
     return output;
@@ -605,7 +614,7 @@ bool DNS::matches_response(const uint8_t* ptr, uint32_t total_sz) const {
 // SOA record
 
 DNS::soa_record::soa_record() 
-: serial_(0), refresh_(0), retry_(0), expire_(0) {
+: serial_(0), refresh_(0), retry_(0), expire_(0), minimum_ttl_(0) {
 
 }
 
