@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,18 +27,18 @@
  *
  */   
 
-#include "dot11/dot11_mgmt.h"
+#include <tins/dot11/dot11_mgmt.h>
 #ifdef TINS_HAVE_DOT11
 
 #include <cstring>
-#include "rsn_information.h"
-#include "memory_helpers.h"
+#include <iterator>
+#include <tins/rsn_information.h>
+#include <tins/memory_helpers.h>
 
 using std::string;
 using std::copy;
 using std::vector;
 using std::back_inserter;
-using std::runtime_error;
 using std::pair;
 using std::make_pair;
 
@@ -239,10 +239,10 @@ void Dot11ManagementFrame::ibss_dfs(const ibss_dfs_params& params) {
 void Dot11ManagementFrame::country(const country_params& params) {
     if ((params.first_channel.size() != params.number_channels.size()) ||
         (params.number_channels.size() != params.max_transmit_power.size())) {
-        throw runtime_error("The length of the lists are distinct");
+        throw invalid_option_value();
     }
     if (params.country.size() != 3) {
-        throw runtime_error("Invalid country identifier length");
+        throw invalid_option_value();
     }
     size_t sz = sizeof(uint8_t) * 3 * params.first_channel.size() + params.country.size();
     // Use 1 byte padding at the end if the length is odd.
