@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
  *
  */
 
-#include "packet_sender.h"
+#include <tins/packet_sender.h>
 #ifndef _WIN32
     #include <sys/socket.h>
     #include <sys/select.h>
@@ -54,25 +54,23 @@
 #endif
 #include <cstring>
 #include <ctime>
-#include <algorithm>
-#include "pdu.h"
-#include "macros.h"
-#include "network_interface.h"
+#include <sstream>
+#include <tins/pdu.h>
+#include <tins/macros.h>
 // PDUs required by PacketSender::send(PDU&, NetworkInterface)
-#include "ethernetII.h"
-#include "radiotap.h"
-#include "dot11/dot11_base.h"
-#include "radiotap.h"
-#include "ieee802_3.h"
-#include "internals.h"
-#include "cxxstd.h"
+#include <tins/ethernetII.h>
+#include <tins/radiotap.h>
+#include <tins/dot11/dot11_base.h>
+#include <tins/radiotap.h>
+#include <tins/ieee802_3.h>
+#include <tins/cxxstd.h>
+#include <tins/detail/pdu_helpers.h>
 #if TINS_IS_CXX11
     #include <chrono>
 #endif // TINS_IS_CXX11
 
 using std::string;
 using std::ostringstream;
-using std::max;
 using std::make_pair;
 using std::vector;
 using std::runtime_error;
@@ -156,6 +154,7 @@ bool PacketSender::ether_socket_initialized(const NetworkInterface& iface) const
     #if defined(BSD) || defined(__FreeBSD_kernel__)
     return ether_socket_.count(iface.id());
     #else
+    Internals::unused(iface);
     return ether_socket_ != INVALID_RAW_SOCKET;
     #endif
 }
@@ -186,13 +185,13 @@ pcap_t* PacketSender::make_pcap_handle(const NetworkInterface& iface) const {
     char error[PCAP_ERRBUF_SIZE];
     pcap_t* handle = pcap_create(TINS_PREFIX_INTERFACE(iface.name()).c_str(), error);
     if (!handle) {
-        throw runtime_error("Error opening pcap handle: " + string(error));
+        throw pcap_error("Error opening pcap handle: " + string(error));
     }
     if (pcap_set_promisc(handle, 1) < 0) {
-        throw runtime_error("Failed to set pcap handle promisc mode: " + string(pcap_geterr(handle)));
+        throw pcap_error("Failed to set pcap handle promisc mode: " + string(pcap_geterr(handle)));
     }
     if (pcap_activate(handle) < 0) {
-        throw runtime_error("Failed to activate pcap handle: " + string(pcap_geterr(handle)));
+        throw pcap_error("Failed to activate pcap handle: " + string(pcap_geterr(handle)));
     }
     return handle;
 }
@@ -234,6 +233,7 @@ void PacketSender::open_l2_socket(const NetworkInterface& iface) {
         }
         ether_socket_[iface.id()] = sock;
     #else
+    Internals::unused(iface);
     if (ether_socket_ == INVALID_RAW_SOCKET) {
         ether_socket_ = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
         
@@ -263,7 +263,10 @@ void PacketSender::open_l3_socket(SocketType type) {
         #else
         typedef const char* option_ptr;
         #endif
-        setsockopt(sockfd, IPPROTO_IP, IP_HDRINCL,(option_ptr)&on,sizeof(on));
+        const int level = (type == IPV6_SOCKET) ? IPPROTO_IPV6 : IPPROTO_IP;
+        if (setsockopt(sockfd, level, IP_HDRINCL, (option_ptr)&on, sizeof(on)) != 0) {
+            throw socket_open_error(make_error_string());
+        }
 
         sockets_[type] = static_cast<int>(sockfd);
     }
@@ -281,6 +284,7 @@ void PacketSender::close_socket(SocketType type, const NetworkInterface& iface)
         }
         ether_socket_.erase(it);
         #elif !defined(_WIN32)
+        Internals::unused(iface);
         if (ether_socket_ == INVALID_RAW_SOCKET) {
             throw invalid_socket_type();
         }
@@ -291,6 +295,7 @@ void PacketSender::close_socket(SocketType type, const NetworkInterface& iface)
         #endif
     }
     else {
+        Internals::unused(iface);
         if (type >= SOCKETS_END || sockets_[type] == INVALID_RAW_SOCKET) {
             throw invalid_socket_type();
         }
@@ -351,17 +356,20 @@ void PacketSender::send_l2(PDU& pdu,
     PDU::serialization_type buffer = pdu.serialize();
 
     #ifdef TINS_HAVE_PACKET_SENDER_PCAP_SENDPACKET
+        Internals::unused(len_addr);
+        Internals::unused(link_addr);
         open_l2_socket(iface);
         pcap_t* handle = pcap_handles_[iface];
         const int buf_size = static_cast<int>(buffer.size());
         if (pcap_sendpacket(handle, (u_char*)&buffer[0], buf_size) != 0) {
-            throw runtime_error("Failed to send packet: " + 
-                                string(pcap_geterr(handle)));
+            throw pcap_error("Failed to send packet: " + string(pcap_geterr(handle)));
         }
     #else // TINS_HAVE_PACKET_SENDER_PCAP_SENDPACKET
         int sock = get_ether_socket(iface);
         if (!buffer.empty()) {
             #if defined(BSD) || defined(__FreeBSD_kernel__)
+            Internals::unused(len_addr);
+            Internals::unused(link_addr);
             if (::write(sock, &buffer[0], buffer.size()) == -1) {
             #else
             if (::sendto(sock, &buffer[0], buffer.size(), 0, link_addr, len_addr) == -1) {
@@ -381,7 +389,7 @@ PDU* PacketSender::recv_l2(PDU& pdu,
                            const NetworkInterface& iface) {
     int sock = get_ether_socket(iface);
     vector<int> sockets(1, sock);
-    return recv_match_loop(sockets, pdu, link_addr, len_addr);
+    return recv_match_loop(sockets, pdu, link_addr, len_addr, false);
 }
 #endif // _WIN32
 
@@ -393,12 +401,12 @@ PDU* PacketSender::recv_l3(PDU& pdu,
     vector<int> sockets(1, sockets_[type]);
     if (type == IP_TCP_SOCKET || type == IP_UDP_SOCKET) {
         #ifdef BSD
-            throw runtime_error("Receiving L3 packets not supported on this platform");
+            throw feature_disabled();
         #endif
         open_l3_socket(ICMP_SOCKET);
         sockets.push_back(sockets_[ICMP_SOCKET]);
     }
-    return recv_match_loop(sockets, pdu, link_addr, len_addr);
+    return recv_match_loop(sockets, pdu, link_addr, len_addr, true);
 }
 
 void PacketSender::send_l3(PDU& pdu, 
@@ -417,7 +425,8 @@ void PacketSender::send_l3(PDU& pdu,
 PDU* PacketSender::recv_match_loop(const vector<int>& sockets, 
                                    PDU& pdu,
                                    struct sockaddr* link_addr,
-                                   uint32_t addrlen) {
+                                   uint32_t addrlen,
+                                   bool is_layer_3) {
     #ifdef _WIN32
         typedef int socket_len_type;
         typedef int recvfrom_ret_type;
@@ -429,10 +438,13 @@ PDU* PacketSender::recv_match_loop(const vector<int>& sockets,
     struct timeval timeout,  end_time;
     int read;
     #if defined(BSD) || defined(__FreeBSD_kernel__)
+        bool is_bsd = true;
         // On* BSD, we need to allocate a buffer using the given size.
-        vector<uint8_t> actual_buffer(buffer_size_);
+        const int buffer_size = is_layer_3 ? 2048 : buffer_size_;
+        vector<uint8_t> actual_buffer(buffer_size);
         uint8_t* buffer = &actual_buffer[0];
     #else
+        bool is_bsd = false;
         uint8_t buffer[2048];
         const int buffer_size = 2048;
     #endif
@@ -445,7 +457,7 @@ PDU* PacketSender::recv_match_loop(const vector<int>& sockets,
         int max_fd = 0;
         for (vector<int>::const_iterator it = sockets.begin(); it != sockets.end(); ++it) {
             FD_SET(*it, &readfds);
-            max_fd = max(max_fd, *it);
+            max_fd = (max_fd > *it) ? max_fd : *it;
         }
         if ((read = select(max_fd + 1, &readfds, 0, 0, &timeout)) == -1) {
             return 0;
@@ -454,7 +466,9 @@ PDU* PacketSender::recv_match_loop(const vector<int>& sockets,
             for (vector<int>::const_iterator it = sockets.begin(); it != sockets.end(); ++it) {
                 if (FD_ISSET(*it, &readfds)) {
                     recvfrom_ret_type size;
-                    #if defined(BSD) || defined(__FreeBSD_kernel__)
+                    // Crappy way of only conditionally running this on BSD + layer2
+                    if (is_bsd && !is_layer_3) {
+                        #if defined(BSD) || defined(__FreeBSD_kernel__)
                         size = ::read(*it, buffer, buffer_size_);
                         const uint8_t* ptr = buffer;
                         // We might see more than one packet
@@ -466,13 +480,15 @@ PDU* PacketSender::recv_match_loop(const vector<int>& sockets,
                             }
                             ptr += BPF_WORDALIGN(bpf_header->bh_hdrlen + bpf_header->bh_caplen);
                         }
-                    #else
+                        #endif // BSD
+                    }
+                    else {
                         socket_len_type length = addrlen;
                         size = ::recvfrom(*it, (char*)buffer, buffer_size, 0, link_addr, &length);
                         if (pdu.matches_response(buffer, size)) {
                             return Internals::pdu_from_flag(pdu.pdu_type(), buffer, size);
                         }
-                    #endif
+                    }
                 }
             }
         }
