@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,11 +30,11 @@
 #ifndef _WIN32
     #include <sys/time.h>
 #endif
-#include <stdexcept>
-#include "packet_writer.h"
-#include "packet.h"
-#include "pdu.h"
-#include "exceptions.h"
+#include <string.h>
+#include <tins/packet_writer.h>
+#include <tins/packet.h>
+#include <tins/pdu.h>
+#include <tins/exceptions.h>
 
 using std::string;
 
@@ -71,11 +71,11 @@ void PacketWriter::write(Packet& packet) {
 
 void PacketWriter::write(PDU& pdu, const struct timeval& tv) {
     PDU::serialization_type buffer = pdu.serialize();
-    struct pcap_pkthdr header = { 
-        tv,
-        static_cast<bpf_u_int32>(buffer.size()),
-        static_cast<bpf_u_int32>(buffer.size())
-    };
+    struct pcap_pkthdr header;
+    memset(&header, 0, sizeof(header));
+    header.ts = tv;
+    header.caplen = static_cast<bpf_u_int32>(buffer.size());
+    header.len = static_cast<bpf_u_int32>(buffer.size());
     pcap_dump((u_char*)dumper_, &header, &buffer[0]);
 }
 
