@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,9 +27,8 @@
  *
  */
  
-#include "pdu.h"
-#include "rawpdu.h"
-#include "packet_sender.h"
+#include <tins/pdu.h>
+#include <tins/packet_sender.h>
 
 using std::swap;
 using std::vector;
@@ -49,12 +48,12 @@ PDU::metadata::metadata(uint32_t header_size, PDUType current_type, PDUType next
 // PDU
 
 PDU::PDU()
-: inner_pdu_() {
+: inner_pdu_(), parent_pdu_() {
 
 }
 
 PDU::PDU(const PDU& other) 
-: inner_pdu_(0) {
+: inner_pdu_(), parent_pdu_() {
     copy_inner_pdu(other);
 }
 
@@ -73,6 +72,9 @@ void PDU::copy_inner_pdu(const PDU& pdu) {
     }
 }
 
+void PDU::prepare_for_serialize() {
+}
+
 uint32_t PDU::size() const {
     uint32_t sz = header_size() + trailer_size();
     const PDU* ptr(inner_pdu_);
@@ -91,9 +93,16 @@ PDU* PDU::recv_response(PacketSender &, const NetworkInterface &) {
     return 0; 
 }
 
+bool PDU::matches_response(const uint8_t* /*ptr*/, uint32_t /*total_sz*/) const {
+    return false;
+}
+
 void PDU::inner_pdu(PDU* next_pdu) {
     delete inner_pdu_;
     inner_pdu_ = next_pdu;
+    if (inner_pdu_) {
+        inner_pdu_->parent_pdu(this);
+    }
 }
 
 void PDU::inner_pdu(const PDU& next_pdu) {
@@ -103,26 +112,33 @@ void PDU::inner_pdu(const PDU& next_pdu) {
 PDU* PDU::release_inner_pdu() {
     PDU* result = 0;
     swap(result, inner_pdu_);
+    if (result) {
+        result->parent_pdu(0);
+    }
     return result;
 }
 
 PDU::serialization_type PDU::serialize() {
     vector<uint8_t> buffer(size());
-    serialize(&buffer[0], static_cast<uint32_t>(buffer.size()), 0);
+    serialize(&buffer[0], static_cast<uint32_t>(buffer.size()));
     return buffer;
 }
 
-void PDU::serialize(uint8_t* buffer, uint32_t total_sz, const PDU* parent) {
+void PDU::serialize(uint8_t* buffer, uint32_t total_sz) {
     uint32_t sz = header_size() + trailer_size();
     // Must not happen...
     #ifdef TINS_DEBUG
     assert(total_sz >= sz);
     #endif
-    prepare_for_serialize(parent);
+    prepare_for_serialize();
     if (inner_pdu_) {
-        inner_pdu_->serialize(buffer + header_size(), total_sz - sz, this);
+        inner_pdu_->serialize(buffer + header_size(), total_sz - sz);
     }
-    write_serialization(buffer, total_sz, parent);
+    write_serialization(buffer, total_sz);
+}
+
+void PDU::parent_pdu(PDU* parent) {
+    parent_pdu_ = parent;
 }
 
 } // Tins
