@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Matias Fontanini
+ * Copyright (c) 2017, Matias Fontanini
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,8 +28,8 @@
  */
 
 #include <limits>
-#include "rawpdu.h"
-#include "tcp_stream.h"
+#include <tins/rawpdu.h>
+#include <tins/tcp_stream.h>
 
 using std::numeric_limits;
 using std::make_pair;
@@ -156,7 +156,7 @@ void TCPStream::safe_insert(fragments_type& frags, uint32_t seq, RawPDU* raw) {
 }
 
 bool TCPStream::generic_process(uint32_t& my_seq,
-                                uint32_t& other_seq, 
+                                uint32_t& /*other_seq*/,
                                 payload_type& pload,
                                 fragments_type& frags,
                                 TCP* tcp) {
