@@ -1,7 +1,7 @@
 #ifndef TINS_DOT11_TEST
 #define TINS_DOT11_TEST
 
-#include "dot11/dot11_base.h"
+#include <tins/dot11/dot11_base.h>
 
 using Tins::Dot11;
 
@@ -16,6 +16,7 @@ inline void test_equals(const Dot11& dot1, const Dot11& dot2) {
     EXPECT_EQ(dot1.more_frag(), dot2.more_frag());
     EXPECT_EQ(dot1.retry(), dot2.retry());
     EXPECT_EQ(dot1.power_mgmt(), dot2.power_mgmt());
+    EXPECT_EQ(dot1.more_data(), dot2.more_data());
     EXPECT_EQ(dot1.wep(), dot2.wep());
     EXPECT_EQ(dot1.order(), dot2.order());
     EXPECT_EQ(dot1.duration_id(), dot2.duration_id());
@@ -31,6 +32,7 @@ inline void test_equals_empty(const Dot11& dot11) {
     EXPECT_EQ(dot11.more_frag(), 0);
     EXPECT_EQ(dot11.retry(), 0);
     EXPECT_EQ(dot11.power_mgmt(), 0);
+    EXPECT_EQ(dot11.more_data(), 0);
     EXPECT_EQ(dot11.wep(), 0);
     EXPECT_EQ(dot11.order(), 0);
     EXPECT_EQ(dot11.duration_id(), 0);
